﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Chat.ChatService;
using System.Windows.Browser;
using System.IO;
using System.Windows.Media.Imaging;
using FluxJpeg.Core;
using FluxJpeg.Core.Encoder;

namespace Chat
{
    public partial class MainPage : UserControl
    {
        EndpointAddress address;
        PollingDuplexHttpBinding binding;
        ChatServiceClient proxy;
        CaptureSource source = new CaptureSource();
        public string User { get; set; }
        public string Partner { set; get; }
        public MainPage()
        {
            InitializeComponent(); 

            this.Loaded+=new RoutedEventHandler(MainPage_Loaded);
            this.txtMessage.KeyDown += new KeyEventHandler(txtMessage_KeyDown);
            this.btnStartVideo.Click += new RoutedEventHandler(btnVideo_Click);
            this.btnStopVideo.Click += new RoutedEventHandler(btnStopVideo_Click);
            this.btnSendVideo.Click += new RoutedEventHandler(btnSendVideo_Click);
            this.btnSend.Click += new RoutedEventHandler(btnSend_Click);            
        }
        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            HtmlElement element;

            element = HtmlPage.Document.GetElementById("lbluser");
            User = element.GetAttribute("innerText");
            element = HtmlPage.Document.GetElementById("lblpartner");
            Partner = element.GetAttribute("innerText");

            this.lblUser.Content = User;
            this.lblPartner.Content = Partner;

            address = new EndpointAddress("http://localhost:8752/ChatService.svc");
            binding = new PollingDuplexHttpBinding(PollingDuplexMode.MultipleMessagesPerPoll);
            proxy = new ChatServiceClient(binding, address);
            proxy.ReceiveReceived += new EventHandler<ReceiveReceivedEventArgs>(proxy_ReceiveReceived);
            proxy.LoginCompleted += new EventHandler<LoginCompletedEventArgs>(proxy_LoginCompleted);
            proxy.LoginAsync(User, Partner);
            AddText("connecting...");
        }

        void proxy_LoginCompleted(object sender, LoginCompletedEventArgs e)
        {
            AddText("connected");
            //连接到服务器后开始接收视频
            ReceiveVideo();
        }

        void proxy_ReceiveReceived(object sender, ReceiveReceivedEventArgs e)
        {
            if (e.Error == null)
            {
                foreach (MessageInfo msg in e.messages)
                {
                    AddText(msg.Sender + "  say:" + msg.Message);
                }
            }
        }

        void ReceiveVideo()
        {
            System.Windows.Threading.DispatcherTimer timerForReceive = new System.Windows.Threading.DispatcherTimer();
            timerForReceive.Interval = new TimeSpan(0, 0, 0, 0, 200);
            timerForReceive.Tick += new EventHandler(timerForReceive_Tick);
            timerForReceive.Start();
            AddText("start to receive video");
        }

        void timerForReceive_Tick(object sender, EventArgs e)
        {
            proxy.GetVideosByteCompleted += new EventHandler<GetVideosByteCompletedEventArgs>(proxy_GetVideosByteCompleted);
            proxy.GetVideosByteAsync(User, Partner);
        }

        void proxy_GetVideosByteCompleted(object sender, GetVideosByteCompletedEventArgs e)
        {
            if(e.Error==null)
            {
                foreach(ChatService.UserVideo video in e.Result)
                {
                    MemoryStream ms = new MemoryStream(video.VideoByte);
                    BitmapImage bitmap = new BitmapImage();
                    bitmap.SetSource(ms);
                    imagePartner.Source = bitmap;
                    ms.Close();
                }
            }
        }

        void btnSendVideo_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Threading.DispatcherTimer timer = new System.Windows.Threading.DispatcherTimer();
            timer.Interval = new TimeSpan(0, 0, 0, 0,200);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();
            AddText("start sending video");
        }

        void timer_Tick(object sender, EventArgs e)
        {
            WriteableBitmap bmp = new WriteableBitmap(this.rectangleUser, null);
            MemoryStream ms = new MemoryStream();
            EncodeJpeg(bmp, ms);
            UserVideo userVideo = new UserVideo();
            userVideo.PartnerName = this.Partner;
            userVideo.UserName = this.User;
            userVideo.VideoByte = ms.GetBuffer();
            proxy.SendVideoByteAsync(userVideo);
            AddText("send a video jpg");
        }

        //编码
        public static void EncodeJpeg(WriteableBitmap bmp, Stream dstStream)
        {
            // Init buffer in FluxJpeg format
            int w = bmp.PixelWidth;
            int h = bmp.PixelHeight;
            int[] p = bmp.Pixels;
            byte[][,] pixelsForJpeg = new byte[3][,]; // RGB colors
            pixelsForJpeg[0] = new byte[w, h];
            pixelsForJpeg[1] = new byte[w, h];
            pixelsForJpeg[2] = new byte[w, h];

            // Copy WriteableBitmap data into buffer for FluxJpeg
            int i = 0;
            for (int y = 0; y < h; y++)
            {
                for (int x = 0; x < w; x++)
                {
                    int color = p[i++];
                    pixelsForJpeg[0][x, y] = (byte)(color >> 16); // R
                    pixelsForJpeg[1][x, y] = (byte)(color >> 8);  // G
                    pixelsForJpeg[2][x, y] = (byte)(color);       // B
                }
            }
            //Encode Image as JPEG
            var jpegImage = new FluxJpeg.Core.Image(new ColorModel { colorspace = ColorSpace.RGB }, pixelsForJpeg);
            var encoder = new JpegEncoder(jpegImage, 95, dstStream);
            encoder.Encode();
        }
        
        //解码
        public static WriteableBitmap DecodeJpeg(Stream srcStream)
        {
            // Decode JPEG
            var decoder = new FluxJpeg.Core.Decoder.JpegDecoder(srcStream);
            var jpegDecoded = decoder.Decode();
            var img = jpegDecoded.Image;
            img.ChangeColorSpace(ColorSpace.RGB);

            // Init Buffer
            int w = img.Width;
            int h = img.Height;
            var result = new WriteableBitmap(w, h);
            int[] p = result.Pixels;
            byte[][,] pixelsFromJpeg = img.Raster;
            // Copy FluxJpeg buffer into WriteableBitmap
            int i = 0;
            for (int x = 0; x < w; x++)
            {
                for (int y = 0; y < h; y++)
                {
                    p[i++] = (0xFF << 24) // A
                                | (pixelsFromJpeg[0][x, y] << 16) // R
                                | (pixelsFromJpeg[1][x, y] << 8)  // G
                                | pixelsFromJpeg[2][x, y];       // B
                }
            }

            return result;
        }

        void btnStopVideo_Click(object sender, RoutedEventArgs e)
        {
            source.Stop();
            AddText("stop video");
        }

        void btnVideo_Click(object sender, RoutedEventArgs e)
        {  
            if (source != null)
            {
                source.Stop();
                source.VideoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice();
                VideoBrush vBrush = new VideoBrush();
                vBrush.SetSource(source);
                this.rectangleUser.Fill = vBrush; 
                if (CaptureDeviceConfiguration.AllowedDeviceAccess || CaptureDeviceConfiguration.RequestDeviceAccess())
                {
                    source.Start();
                }

                AddText("start video");
            } 
        }

        void txtMessage_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                SendMsg();
            }
        }        

        
        void btnSend_Click(object sender, RoutedEventArgs e)
        {
            SendMsg();
        }

        void SendMsg()
        {
            MessageInfo msgInfo = new MessageInfo();
            msgInfo.ID = Guid.NewGuid().ToString();
            msgInfo.Message = this.txtMessage.Text;
            msgInfo.ReceiveUser = Partner;
            msgInfo.Sender = User;
            msgInfo.SendTime = DateTime.Now;
            msgInfo.State = 0;
            proxy.SendMessageCompleted += new EventHandler<SendMessageCompletedEventArgs>(proxy_SendMessageCompleted);
            proxy.SendMessageAsync(msgInfo);
            this.txtMessage.Text = "";
        }

        void proxy_SendMessageCompleted(object sender, SendMessageCompletedEventArgs e)
        {
            this.txtMessage.Text = "";
        }

        void AddText(string strText)
        {
            this.txtChatPlant.Text +=DateTime.Now.ToLongTimeString()+ " "+strText+Environment.NewLine;
        }

    }
}
